﻿////------------------------------------------------------------------------------------------
//
//    MindFusion Office Floorplan Designer 
//     Copyright (c) 2012, MindFusion LLC - Bulgaria
//
////------------------------------------------------------------------------------------------

using MindFusion.Diagramming;
using MindFusion.Diagramming.WinForms;
using MindFusion.Diagramming.WinForms.Behaviors;

using System.Drawing;


namespace FloorplanDesigner
{
	/// <summary>
	/// The FloorplanBehavior class is derived from the MindFusion BehaviorBase class, and is used to 
	/// provide information to the DiagramView control about how mouse movements and actions are to 
	/// be interpreted.
	/// </summary>
	public class FloorplanBehavior : BehaviorBase
	{
		#region Private variables

		private readonly FormMain _formMain;

		#endregion Private variables


		#region Constructor

		/// <summary>
		/// Constructor.
		/// </summary>
		public FloorplanBehavior(FormMain formMain, DiagramView diagramView)
			: base(diagramView)
		{
			_formMain = formMain;
		}

		#endregion Constructor


		#region Public override methods needed by BehaviorBase

		/// <summary>
		/// Override for the SetMouseCursor() method, used to set the mouse cursor (and indirectly the 
		/// behavior if the mouse is clicked?) depending on where the mouse is. This gets called many, 
		/// many times, for example when the mouse is simply being moved over the view area.
		/// </summary>
		public override CursorHint SetMouseCursor(PointF currentPoint, out bool startInteraction)
		{
			// Get indication of whether the cursor is on an existing floorplan item and near a handle
			EHandleId adjustmentHandle;
			GetNodeAndHandle(currentPoint, out adjustmentHandle);

			// If cursor is near a handle set the mouse cursor to hint of what can be done
			startInteraction = true;  // Assume cursor is near a handle
			switch (adjustmentHandle)
			{
				case EHandleId.eTopLeft: return CursorHint.DiagonalResize;
				case EHandleId.eTopRight: return CursorHint.CounterDiagonalResize;
				case EHandleId.eBottomRight: return CursorHint.DiagonalResize;
				case EHandleId.eBottomLeft: return CursorHint.CounterDiagonalResize;
				case EHandleId.eTopCenter: return CursorHint.VerticalResize;
				case EHandleId.eRightCenter: return CursorHint.HorizontalResize;
				case EHandleId.eBottomCenter: return CursorHint.VerticalResize;
				case EHandleId.eLeftCenter: return CursorHint.HorizontalResize;
				case EHandleId.eMoveHandle: return CursorHint.Move;
				case EHandleId.eRotateHandle: return CursorHint.Rotate;
			}

			// Cursor not on an existing floorplan item and near a handle, do default return
			startInteraction = false;
			return CursorHint.Pointer;
		}


		/// <summary>
		/// Override for method StartDraw(). This has to determine if a new floorplan item (external 
		/// or internal wall, table or desk) is to be drawn or a pseudo-item (door) is to be "placed" 
		/// or if an existing item should be modified or moved.
		/// </summary>
		public override InteractionState StartDraw(PointF startPoint)
		{
			// Get indication of whether the cursor is on an existing floorplan item and near a handle
			EHandleId adjustmentHandle;
			NodeItemBase nodeItem = GetNodeAndHandle(startPoint, out adjustmentHandle);

			// If cursor is on an existing item and on or near a handle, perform the implied action
			if (adjustmentHandle != EHandleId.eUnknown)
				return new InteractionState(nodeItem,
					new NodeAdjustmentHandle((NodeHandleType)adjustmentHandle), Action.Modify);

			// This code only here because of the "pseudo margins" - can be removed if/when deprecated 
			if (startPoint.X < FormMain.CPseudoMarginZero || startPoint.Y < FormMain.CPseudoMarginZero)
				return new InteractionState(null, null, Action.None);

			// Cursor not on an existing floorplan item and near a handle, start creating a new 
			//  floorplan item, as specified on the ToolStrip
			nodeItem = _formMain.GetNewNode(startPoint, ENodeRequirement.eDrawable);
			if (nodeItem != null)
				return new InteractionState(nodeItem, null, Action.Create);

			// No ToolStrip button checked, nothing can be done
			return new InteractionState(null, null, Action.None);
		}

		#endregion Public override methods needed by BehaviorBase


		#region Private methods

		/// <summary>
		/// Method used by the above two methods to find the floorplan item the mouse is over (if 
		/// any), and if so to identify which handle the mouse is over or very near, if any.
		/// </summary>
		private NodeItemBase GetNodeAndHandle(PointF mousePoint, out EHandleId handleId)
		{
			handleId = EHandleId.eUnknown;  // Assume mouse not over a handle

			// Test if mouse is over a floorplan item, return null if not
			NodeItemBase nodeItemBase =
						Diagram.GetNodeAt(mousePoint, NodeItemBase.CNearnessThreshold) as NodeItemBase;
			if (nodeItemBase == null)
				return null;

			// Get the adjustment handle if cursor is over or near a handle
			NodeAdjustmentHandle adjustmentHandle = nodeItemBase.HitTestHandle(mousePoint) as NodeAdjustmentHandle;
			if (adjustmentHandle != null)
				handleId = (EHandleId)adjustmentHandle.HandleType;

			return nodeItemBase;
		}

		#endregion Private methods

	}
}