﻿////------------------------------------------------------------------------------------------
//
//    MindFusion Office Floorplan Designer 
//     Copyright (c) 2012, MindFusion LLC - Bulgaria
//
////------------------------------------------------------------------------------------------

using System;
using System.IO;
using System.Windows.Forms;


namespace FloorplanDesigner
{
   /// <summary>
   /// This is a modal form used to prompt the user to enter some "settings" for an office 
   /// floorplan. It is used for the File - New and View - Floorplan Settings menu items.
   /// </summary>
   public partial class FormSettings : Form
   {
      #region Private variables

      // Reference to the current (perhaps brand new) FloorplanDiagram object. This is used as both 
      //  the source for the settings and the place where the settings are saved.
      private readonly FloorplanDiagram _floorplanDiagram;

      #endregion Private variables


      #region Constructor

      /// <summary>
      /// Constructor.
      /// </summary>
      public FormSettings(FloorplanDiagram floorplanDiagram, bool isNew)
      {
         // Initialize the Visual Studio designer-generated stuff, save the argument
         InitializeComponent();
         _floorplanDiagram = floorplanDiagram;

         // Set the header text
         if (isNew)
            lblHeader.Text = "Specify the settings for the new floorplan:";
         else
            lblHeader.Text = "Current settings for the floorplan:";

         // Copy the settings from the floorplan object to the controls
         switch (floorplanDiagram.RealWorldUnits)
         {
            case ERealWorldUnits.eFeet:    rbFeet.Checked = true;    break;
            case ERealWorldUnits.eMeters:  rbMeters.Checked = true;  break;
         }
         tbScalingFactor.Text = floorplanDiagram.ScalingFactor.ToString();
         if (floorplanDiagram.RealWorldSizeX > 0 & floorplanDiagram.RealWorldSizeY > 0)
         {
            tbSizeX.Text = floorplanDiagram.RealWorldSizeX.ToString();
            tbSizeY.Text = floorplanDiagram.RealWorldSizeY.ToString();
         }
         tbBackgroundImageFile.Text = floorplanDiagram.BackgroundImageFile;
         tbBackgroundScaling.Text = floorplanDiagram.BackgroundScaling.ToString();
         tbBackgroundOpacity.Text = floorplanDiagram.BackgroundOpacity.ToString();
      }

      #endregion Constructor

      
      #region Private methods

      /// <summary>
      /// Method called when the user (or the program) changes the feet vs. meters radio buttons. 
      /// This tries to adjust the scaling factor to correspond to the change.
      /// </summary>
      private void FeetVsMeters_CheckedChanged(object sender, EventArgs e)
      {
         if (tbScalingFactor.Text == "-1")
            return;  // Ignore setup from constructor

         float f;
         if (float.TryParse(tbScalingFactor.Text, out f))
         {
            if (rbFeet.Checked)
               tbScalingFactor.Text = ((int)((f * (float)FloorplanDiagram.CDefaultScalingForFeet / 
                               (float)FloorplanDiagram.CDefaultScalingForMeters) + 0.5)).ToString();
            else 
               tbScalingFactor.Text = ((int)((f * (float)FloorplanDiagram.CDefaultScalingForMeters / 
                               (float)FloorplanDiagram.CDefaultScalingForFeet) + 0.5)).ToString();
         }
         else
         {
            if (rbFeet.Checked)
               tbScalingFactor.Text = FloorplanDiagram.CDefaultScalingForFeet.ToString();
            else 
               tbScalingFactor.Text = FloorplanDiagram.CDefaultScalingForMeters.ToString();
         }
      }


      /// <summary>
      /// Method called when user clicks on the background image "Browse" button. This uses the Open 
      /// File dialog to get a filename, and checks that the file exists, but it does not read the 
      /// file or apply the background image to the floorplan.
      /// </summary>
      private void BackgroundBrowse_Click(object sender, EventArgs e)
      {
         // Display an open file dialog to get the full filename of the background image
         using (OpenFileDialog openFileDialog = new OpenFileDialog())
         {
            openFileDialog.Title = "Open Background Image File";
            openFileDialog.InitialDirectory =
                                       Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            openFileDialog.Filter = 
               "Image Files (*.bmp;*.gif;*.jpg;*.png;*.tif)|*.bmp;*.gif;*.jpg;*.png;*.tif|" + 
               "All Files (*.*)|*.*";
            DialogResult dialogResult = openFileDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
               tbBackgroundImageFile.Text = openFileDialog.FileName;
            else 
               return;
         }

         // Check that the file exists (but don't try to open it at this point)
         if (!File.Exists(tbBackgroundImageFile.Text))
            MessageBox.Show("Specified file does not exist.", FormMain.CProgramName, 
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
      }


      /// <summary>
      /// Method called when user clicks on the "OK" button, or hits the Enter key. This just copies 
      /// the settings from the controls on the form to the floorplan object, and then closes the 
      /// form. 
      /// </summary>
      private void Ok_Click(object sender, EventArgs e)
      {
         if (rbFeet.Checked)
            _floorplanDiagram.RealWorldUnits = ERealWorldUnits.eFeet;
         else 
            _floorplanDiagram.RealWorldUnits = ERealWorldUnits.eMeters;

         _floorplanDiagram.ScalingFactor = GetIntFromTextBox(tbScalingFactor, -1);

         _floorplanDiagram.RealWorldSizeX = GetIntFromTextBox(tbSizeX, 0);
         _floorplanDiagram.RealWorldSizeY = GetIntFromTextBox(tbSizeY, 0);

         _floorplanDiagram.BackgroundImageFile = tbBackgroundImageFile.Text.Trim();
         _floorplanDiagram.BackgroundScaling = GetIntFromTextBox(tbBackgroundScaling, 100);
         _floorplanDiagram.BackgroundOpacity = 
                 GetIntFromTextBox(tbBackgroundOpacity, FloorplanDiagram.CDefaultBackgroundOpacity);

         Close();
      }


      /// <summary>
      /// Method to parse a text box's text field and return an int value.
      /// </summary>
      /// <returns>int value if parse works, else default value</returns>
      private static int GetIntFromTextBox(TextBox textBox, int defaultValue)
      {
         int i;
         if (int.TryParse(textBox.Text.Trim(), out i))
            return i;
         return defaultValue;
      }


      /// <summary>
      /// Method called when user clicks on the "Cancel" button, or hits the Esc key.
      /// </summary>
      private void Cancel_Click(object sender, EventArgs e)
      {
         Close();
      }

      #endregion Private methods

   }
}
