﻿////------------------------------------------------------------------------------------------
//
//    MindFusion Office Floorplan Designer 
//     No copyright - borrowed from various sources
//
////------------------------------------------------------------------------------------------

using System;
using System.Diagnostics.CodeAnalysis;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;


namespace FloorplanDesigner
{
   public static class ImageUtilities
   {
      #region Public static methods

      /// <summary>
      /// Overload for following method to do a simple resize without changing the aspect ratio, 
      /// maybe also changing the opacity.
      /// </summary>
      /// <param name="inputImage">input Image object</param>
      /// <param name="resizeFactor">amount of resizing wanted</param>
      /// <param name="opacityFactor">to modify result's opacity, 1.0 = no change</param>
      /// <returns>resized image  - will be input image if resizeFactor = 1.0</returns>
      public static Image ResizeImage(Image inputImage, float resizeFactor, float opacityFactor)
      {
         if (resizeFactor == 1f && opacityFactor == 1f)
            return inputImage;

         return ResizeImage(inputImage, (int)(resizeFactor * inputImage.Width), 
                            (int)(resizeFactor * inputImage.Height), false, true, opacityFactor);
      }


      /// <summary>
      /// Method to resize and/or crop an Image object to the specified size, referred to as the new 
      /// "frame" in this method. An "opacity" can also be applied, so the result looks semi-
      /// transparent. 
      /// 
      /// If the aspect ratio of the input image does not match the aspect ratio of the desired 
      /// "frame", then the cropOk option comes into play. If this option is on then the top and 
      /// bottom or the left and right of the original image are discarded to make the image fit the 
      /// frame. If the option is off then black borders are added to the top and bottom or to the 
      /// left and right.
      /// 
      /// If the input image is smaller than the desired "frame", then the upsizeOk option comes 
      /// into play. If this option is on then the image is resized upwards, and the cropOk option 
      /// will also come into play if the aspect ratio is "wrong". If upsizeOk is off then the input 
      /// image is placed in the middle of the frame with black borders on all four sides.
      /// 
      /// </summary>
      /// <param name="inputImage">input Image object</param>
      /// <param name="frameWidth">width of the desired image</param>
      /// <param name="frameHeight">height of the desired image</param>
      /// <param name="cropOk">see comments above</param>
      /// <param name="upsizeOk">see comments above</param>
      /// <param name="opacityFactor">to modify result's opacity, 1.0 = no change</param>
      /// <returns>image of the specified dimensions - may be input image if its dimensions were OK</returns>
      [SuppressMessage("Microsoft.Reliability", "CA2000:Dispose objects before losing scope")]
      public static Image ResizeImage(Image inputImage, int frameWidth, int frameHeight, 
                                      bool cropOk, bool upsizeOk, float opacityFactor)
      {
         // Check resizing needed, just return a reference to the existing image if not
         if (inputImage.Width == frameWidth && inputImage.Height == frameHeight && 
             opacityFactor == 1f)
            return inputImage;

         // Create the objects we need to work with (new image background will be black)
         Image newImage = new Bitmap(frameWidth, frameHeight, PixelFormat.Format24bppRgb);
         Graphics graphicsObject = Graphics.FromImage(newImage);
         //gObject.Clear(Color.Black);  // Not necessary with PixelFormat.Format24bppRgb

         // These from recommendations found here: 
         //  http://blog.dmbcllc.com/2009/07/30/net-image-scaling-in-csharp/
         graphicsObject.CompositingMode = CompositingMode.SourceOver;
         graphicsObject.CompositingQuality = CompositingQuality.HighQuality;
         graphicsObject.InterpolationMode = InterpolationMode.HighQualityBicubic;
         graphicsObject.PixelOffsetMode = PixelOffsetMode.HighQuality;
         graphicsObject.SmoothingMode = SmoothingMode.HighQuality;

         // Compute the amount of scaling that would be needed to resize the two dimensions
         float reSizeX = frameWidth / (float)inputImage.Width;
         float reSizeY = frameHeight / (float)inputImage.Height;

         // If cropping option has been selected we choose the larger value, if not we choose the 
         //  smaller value
         float resizeScale;
         if (cropOk)
            resizeScale = Math.Max(reSizeX, reSizeY);
         else
            resizeScale = Math.Min(reSizeX, reSizeY);

         // If the upsizeOk option is off then we don't resize upwards so max resize scaling is 1.0
         if (!upsizeOk)
            resizeScale = Math.Min(resizeScale, 1);

         // Compute the size of the image after resizing
         int newWidth = (int)(inputImage.Width * resizeScale);  // Maybe wider than frame
         int newHeight = (int)(inputImage.Height * resizeScale);  // Maybe higher than frame

         // Compute the position at which to place the resized image on the frame
         int newX = (frameWidth - newWidth) / 2;  // Maybe outside frame
         int newY = (frameHeight - newHeight) / 2;  // Maybe outside frame

         // Sufficient information is now available to resize and/or crop the input image while 
         //  drawing it onto the new bitmap-generated graphics object, which is our frame

         // Test for no opacity manipulation and use a simple call to DrawImage if so
         if (opacityFactor == 1f)
            graphicsObject.DrawImage(inputImage, newX, newY, newWidth, newHeight);
         else
         {
            // The opacity of the image is to be changed, which can be done at the same time as the 
            //  resizing (assuming resizing is in fact being done)

            // Setting opacity looks strange if the background isn't white, so create a white 
            //  rectangle where the (possibly resized) image will be rendered
            graphicsObject.FillRectangle(new SolidBrush(Color.White), 
                                         newX, newY, newWidth, newHeight);

            // Create a ColorMatrix object and specify the opacity
            ColorMatrix colorMatrix = new ColorMatrix();
            colorMatrix.Matrix33 = opacityFactor;

            // Create an ImageAttributes object and link it to the ColorMatrix object 
            ImageAttributes imageAttributes = new ImageAttributes();       
            imageAttributes.SetColorMatrix(colorMatrix, ColorMatrixFlag.Default, 
                                           ColorAdjustType.Bitmap);
 
            // Call DrawImage, using all of the bells and whistles to do both resizing (maybe) and 
            //  applying the modified opacity
            graphicsObject.DrawImage(inputImage, new Rectangle(newX, newY, newWidth, newHeight), 
                    0, 0, inputImage.Width, inputImage.Height, GraphicsUnit.Pixel, imageAttributes); 
         }

         // All done, dispose of the graphics object and return the new image
         graphicsObject.Dispose();
         return newImage;         
      }

      #endregion Public static methods

   }
}
