﻿////------------------------------------------------------------------------------------------
//
//    MindFusion Office Floorplan Designer 
//     Copyright (c) 2012, MindFusion LLC - Bulgaria
//
////------------------------------------------------------------------------------------------

using MindFusion.Diagramming;
using MindFusion.Drawing;

using System.Collections.Generic;
using System.Drawing;

using SystemPen = System.Drawing.Pen;


namespace FloorplanDesigner
{
   /// <summary>
   /// This is a base class from which all floorplan "items" are derived. It, in turn, is derived 
   /// from the MindFusion DiagramNode class, and represents a "custom item". This is described on 
   /// the MindFusion Developer's Guide in the tutorial section "Creating Custom Node Types", and is 
   /// illustrated in the Fishbone sample program.
   /// </summary>
   public abstract class NodeItemBase : DiagramNode
   {
      #region Constants

      // Mouse must be this close to a handle to be close enough. Dropping a door must be within 
      //  three times this distance to a wall.
      internal const float CNearnessThreshold = 4f;

      #endregion Constants


      #region Constructors

      // Paramerterless constructor, needed for several purposes
      protected NodeItemBase()
      {}


      // Copy constructor, needed for copy to/from clipboard
      protected NodeItemBase(NodeItemBase copyFrom)
         : base(copyFrom)
      {}

      #endregion Constructors

      
      #region Public and protected override methods

      /// <summary>
      /// Override for the AllowCreate() method. This is apparently needed to avoid the mouse cursor 
      /// showing a "no entry" symbol when the mouse is over part of the floorplan item during 
      /// creation, resulting in the item disappearing when the left mouse button is released.
      /// </summary>
      protected override bool AllowCreate(PointF currentPoint, InteractionState interactionState)
      {
         return true;
      }


      /// <summary>
      /// Override for the AllowModify() method, which simply indicates that all floreplan items can 
      /// be created anywhere.
      /// </summary>
      protected override bool AllowModify(PointF currentPoint, InteractionState interactionState)
      {
         return true;
      }

      
      /// <summary>
      /// Override for the DrawShadow() method is empty - shadowing not supported.
      /// </summary>
      public override void DrawShadow(IGraphics iGraphics, RenderOptions renderOptions)
      {}

      #endregion Public and protected override methods


      #region Protected methods 

      /// <summary>
      /// Method to draw one or more handles, as specified by a List{} of points. These are intended 
      /// to resemble the handles drawn by the MindFusion code. 
      /// </summary>
      protected static void DrawHandles(List<PointF> handlesList, IGraphics iGraphics, 
                                        HandlesVisualStyle handlesVisualStyle, float handleSize)
      {
         float halfHandle = handleSize / 2f;

         using (SystemPen handlePen = handlesVisualStyle.HandlePen.CreateGdiPen(), 
                          whitePen = new SystemPen(Color.White, handleSize))
         {
            foreach (PointF handlePoint in handlesList)
            {
               iGraphics.DrawLine(whitePen, handlePoint.X - halfHandle, handlePoint.Y, 
                                            handlePoint.X + halfHandle, handlePoint.Y);

               iGraphics.DrawRectangle(handlePen, handlePoint.X - halfHandle, 
                                       handlePoint.Y - halfHandle, handleSize, handleSize);
            }
         }
      }


      /// <summary>
      /// Method to test if the mouse is near enough to a point to be considered a hit.
      /// </summary>
      protected static bool TestForNear(PointF mousePoint, float positionX, float positionY)
      {
         if (mousePoint.X < positionX - CNearnessThreshold || 
             mousePoint.X > positionX + CNearnessThreshold)
            return false;
         
         if (mousePoint.Y < positionY - CNearnessThreshold || 
             mousePoint.Y > positionY + CNearnessThreshold)
            return false;

         return true;
      }

      #endregion Protected methods


      #region Abstract properties

      // Property indicating if this item can be drawn on the drawing area
      abstract public bool IsDrawable {get;}

      // Property indicating if this item can simply be dropped on the drawing area
      abstract public bool IsDroppable {get;}

      #endregion Abstract properties
      
   }
}
