﻿////------------------------------------------------------------------------------------------
//
//    MindFusion Office Floorplan Designer 
//     Copyright (c) 2012, MindFusion LLC - Bulgaria
//
////------------------------------------------------------------------------------------------

using MindFusion.Diagramming;
using MindFusion.Drawing;

using System.Drawing;
using System.IO;
using System.Xml;

using SystemBrush = System.Drawing.Brush;


namespace FloorplanDesigner
{
   /// <summary>
   /// This class implements a floorplan item of type "Text Box".
   /// </summary>
   public class NodeTextBox : NodeItemBase, InplaceEditable
   {
      #region Constants

      // XML element name for the added field
      private const string CPlainText = "PlainText";

      // Default size of the node
      const float CDefaultWidth = 100f; 
      const float CDefaultHeight = 30f;
      const float CHalfWidth = CDefaultWidth / 2f;
      const float CHalfHeight = CDefaultHeight / 2f;

      #endregion Constants


      #region Private variables

      // Text shown by the Text Box node
      private string _plainText = "Double-click to enter text";

      #endregion Private variables


      #region Constructors

      // Paramerterless constructor, needed for deserialization and copy from clipboard
      public NodeTextBox()
      {}

      
      // Copy constructor, needed for copy to/from clipboard
      public NodeTextBox(NodeTextBox copyFrom)
         : base(copyFrom)
      {
         _plainText = copyFrom._plainText;
      }


      // Constructor for when user creates an item on the diagram area
      public NodeTextBox(PointF startPoint)
      {
         base.Font = new Font("Microsoft Sans Serif", 8);

         // Set initial default size of item unless this is a deserialization call
         if (startPoint != PointF.Empty)
         {
            base.Bounds = RectangleF.FromLTRB(startPoint.X - CHalfWidth, startPoint.Y - CHalfHeight,
                                             startPoint.X + CHalfWidth, startPoint.Y + CHalfHeight);
         }
      }

      #endregion Constructors


      #region Public and protected override methods

      /// <summary>
      /// Override for the Draw() method to render the text for this node.
      /// </summary>
      public override void Draw(IGraphics iGraphics, RenderOptions renderOptions)
      {
         iGraphics.DrawString(_plainText, base.Font, Brushes.Black, base.Bounds);
      }


      /// <summary>
      /// Override for the LoadFrom() method, needed to support loading from binary disk files 
      /// (currently not used), and also copying from the clipboard.
      /// </summary>
      protected override void LoadFrom(BinaryReader binaryReader, PersistContext persistContext)
      {
         base.LoadFrom(binaryReader, persistContext);

         _plainText = binaryReader.ReadString();
      }


      /// <summary>
      /// Override for the LoadFrom() method, needed to support copying XML disk files.
      /// </summary>
      protected override void LoadFromXml(XmlElement xmlElement, XmlPersistContext persistContext)
      {
         base.LoadFromXml(xmlElement, persistContext);
         
         _plainText = persistContext.ReadString(CPlainText, xmlElement);
      }


      /// <summary>
      /// Override for the SaveTo() method, needed to support saving to binary disk files (currently 
      /// not used), and also copying to the clipboard.
      /// </summary>
      protected override void SaveTo(BinaryWriter binaryWriter, PersistContext persistContext)
      {
         base.SaveTo(binaryWriter, persistContext);

         binaryWriter.Write(_plainText);
      }


      /// <summary>
      /// Override for the SaveToXml() method, needed to support saving to XML disk files.
      /// </summary>
      protected override void SaveToXml(XmlElement xmlElement, XmlPersistContext persistContext)
      {
         base.SaveToXml(xmlElement, persistContext);

         persistContext.WriteString(_plainText, CPlainText, xmlElement);
      }

      #endregion Public and protected override methods


      // Methods required by the InplaceEditable interface. These implement letting the user modify 
      //  the text associated with the node.

      public RectangleF GetEditRect(DiagramItem diagramItem, PointF mousePosition)
      {
         return base.Bounds;
      }

      public string GetTextToEdit()
      {
         return _plainText;
      }

      public void SetEditedText(string newText)
      {
         _plainText = newText;
      }

      
      // Properties required by the NodeItemBase class. This node can be either "drawn" or "dropped"
      //  on the floorplan.

      public override bool IsDrawable
      { get { return true; }}

      public override bool IsDroppable
      { get { return true; }}
   }
}
